require 'test_helper'

class RadioTest < ActiveSupport::TestCase

  def assigns
    {person: ::Person.new}
  end

  def helpers
    mock_action_view do
      def people_path
        '/people'
      end

      def employees_path
        '/employees'
      end
    end
  end

  # TODO Still failing
  # test "radio requires a parent component" do
  #   assert_match "Error rendering ExpressTemplates::Components::Forms::Radio", arbre {
  #     radio :preferred_email_format, options: ['HTML', 'Text']
  #   }
  # end

  def radio_with_array_options
    arbre {
      express_form(:person) {
        radio :preferred_email_format, options: ['HTML', 'Text']
      }
    }
  end

  test "radio has correct label field name and text" do
    assert_match(/<label for="person_preferred_email_format"/, radio_with_array_options)
  end

  test "radio options present with class 'radio'" do
    assert_match(/<input.*class="radio"/, radio_with_array_options)
  end

  def radio_with_hash_options
    arbre {
      express_form(:person) {
        radio :subscribed, options: {1 => 'Yes', 0 => 'No'}, label_wrapper_class: 'my-wrapper'
      }
    }
  end

  test "radio options may be specified with a hash" do
    compiled = radio_with_hash_options
    assert_match('<label class="my-wrapper">', compiled)
    assert_match('input class="radio" type="radio" value="0" name="person[subscribed]" id="person_subscribed_0" />No', compiled)
    assert_match('input class="radio" type="radio" value="1" name="person[subscribed]" id="person_subscribed_1" />Yes', compiled)
  end

  # TODO: Still failing
  # test "radio displays error if given improper options" do
  #   assert_match "Error rendering ExpressTemplates::Components::Forms::Radio component: No association collection for: person.subscribed", arbre {
  #     express_form(:person) {
  #       radio :subscribed, "Garbage options"
  #     }
  #   }
  # end

  def radio_with_options_omitted
    arbre(employee: resource) {
      express_form(:employee) {
        radio :department_id
      }
    }
  end

  class ::Department < ::Gender
    def self.order(*)
      all
    end
    def self.all
      return [new(1, 'Accounting'), new(2, 'Marketing')]
    end
  end
  class ::Employee
    def self.reflect_on_association(name)
      if name.eql? :department
        dummy_association = Object.new
        class << dummy_association
          def macro ; :belongs_to ; end
          def klass ; ::Department ; end
          def polymorphic? ; false ; end
        end
        return dummy_association
      end
    end
  end

  test "radio options from collection when options omitted" do
    assert_match(/input type="radio" value="1" name="employee\[department_id\]" id="employee_department_id_1"/, radio_with_options_omitted)
  end

  test "label_after is true, options specified is an array - label appears after the input" do
    html = arbre {
      express_form(:person) {
        radio :subscribed, options: ['Yes', 'No', 'IThinkSo'], label_after: true
      }
    }

    document = Nokogiri::HTML.parse(html)
    radio_input_containers = document.css(".radio-input-container")

    container_1 = radio_input_containers[0]
    assert_equal "input", container_1.element_children[0].name
    assert_equal "label", container_1.element_children[1].name

    container_2 = radio_input_containers[1]
    assert_equal "input", container_2.element_children[0].name
    assert_equal "label", container_2.element_children[1].name

    # Label's for value is downcased
    container_3 = radio_input_containers[2]
    for_attr = container_3.element_children[1].attributes['for']
    assert_equal "person_subscribed_ithinkso", for_attr.value
  end

  test "label_after is true, options specified is a hash - label appears after the input" do
    html = arbre {
      express_form(:person) {
        radio :subscribed, options: {0 => 'Yes', 1 => 'No'}, label_after: true
      }
    }

    document = Nokogiri::HTML.parse(html)
    radio_input_containers = document.css(".radio-input-container")

    container_1 = radio_input_containers[0]
    assert_equal "input", container_1.element_children[0].name
    assert_equal "label", container_1.element_children[1].name

    container_2 = radio_input_containers[1]
    assert_equal "input", container_2.element_children[0].name
    assert_equal "label", container_2.element_children[1].name

    # Label's for value is downcased
    for_attr = container_2.element_children[1].attributes['for']
    assert_equal "person_subscribed_1", for_attr.value
  end

  # test "radio supports html options"

  # test "html_options passed correctly when collection is omitted"

  # test "radio helper options passed to collection_radio_buttons"
end
